/* /////////////////////////////////////////////////////////////////////////////
 * File:    cstring/cstring.h
 *
 * Purpose: Definition of the cstring API.
 *
 * Created: 16th June 1994
 * Updated: 29th July 2005
 *
 * Home:    http://synesis.com.au/software/
 *
 * Copyright 1994-2005, Matthew Wilson and Synesis Software
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice, this
 *   list of conditions and the following disclaimer. 
 * - Redistributions in binary form must reproduce the above copyright notice,
 *   this list of conditions and the following disclaimer in the documentation
 *   and/or other materials provided with the distribution.
 * - Neither the names of Matthew Wilson and Synesis Software nor the names of
 *   any contributors may be used to endorse or promote products derived from
 *   this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 * POSSIBILITY OF SUCH DAMAGE.
 *
 * ////////////////////////////////////////////////////////////////////////// */


/** \file cstring/cstring.h Definition of the cstring API
 */

#ifndef CSTRING_INCL_CSTRING_H_CSTRING_INCL_H_CSTRING
#define CSTRING_INCL_CSTRING_H_CSTRING_INCL_H_CSTRING

/* /////////////////////////////////////////////////////////////////////////////
 * Version
 */

#ifndef CSTRING_DOCUMENTATION_SKIP_SECTION
# define CSTRING_VER_CSTRING_H_CSTRING_MAJOR    3
# define CSTRING_VER_CSTRING_H_CSTRING_MINOR    2
# define CSTRING_VER_CSTRING_H_CSTRING_REVISION 1
# define CSTRING_VER_CSTRING_H_CSTRING_EDIT     44
#endif /* !CSTRING_DOCUMENTATION_SKIP_SECTION */

/** \def CSTRING_VER_MAJOR
 * The major version number of cstring
 */

/** \def CSTRING_VER_MINOR
 * The minor version number of cstring
 */

/** \def CSTRING_VER_REVISION
 * The revision version number of cstring
 */

/** \def CSTRING_VER
 * The current composite version number of cstring
 */

#define CSTRING_VER_MAJOR       3
#define CSTRING_VER_MINOR       2
#define CSTRING_VER_REVISION    1

#ifndef CSTRING_DOCUMENTATION_SKIP_SECTION
# define CSTRING_VER_1_0_1       0x01000100
# define CSTRING_VER_1_0_2       0x01000200
# define CSTRING_VER_2_0_1       0x02000100
# define CSTRING_VER_2_1_1       0x02010100
# define CSTRING_VER_2_2_1       0x02020100
# define CSTRING_VER_3_0_1       0x03000100
# define CSTRING_VER_3_1_1       0x03010100
# define CSTRING_VER_3_1_2       0x03010200
# define CSTRING_VER_3_1_3       0x03010300
# define CSTRING_VER_3_2_1       0x03020100

# define CSTRING_VER             CSTRING_VER_3_2_1
#else /* ? CSTRING_DOCUMENTATION_SKIP_SECTION */
# define CSTRING_VER             0x03020100
#endif /* !CSTRING_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Includes
 */

#include <stddef.h>

/* /////////////////////////////////////////////////////////////////////////////
 * Storage class
 */

#ifdef __cplusplus
# define CSTRING_EXTERN_C       extern "C"
#else /* ? __cplusplus */
# define CSTRING_EXTERN_C       extern
#endif /* ? __cplusplus */

/* /////////////////////////////////////////////////////////////////////////////
 * Constants & definitions
 */

/** \brief The return code type of the cstring API
 */
enum CSTRING_RC
{
        CSTRING_RC_SUCCESS               =   0  /*!< Operation completed successfully */
    ,   CSTRING_RC_OUTOFMEMORY                  /*!< Out of memory */
    ,   CSTRING_RC_FIXED                        /*!< Operation cannot procede because the cstring has fixed capacity */
    ,   CSTRING_RC_BORROWED                     /*!< Operation cannot procede because the cstring is borrowed */
    ,   CSTRING_RC_READONLY                     /*!< Operation cannot procede because the cstring is readonly */
    ,   CSTRING_RC_INVALIDARENA                 /*!< Invalid arena */
    ,   CSTRING_RC_CUSTOMARENANOTSUPPORTED      /*!< Custom arena functionality not currently supported */
    ,   CSTRING_RC_EXCEEDFIXEDCAPACITY          /*!< Operation cannot procede because the current capacity would be exceeded and the cstring has fixed capacity */
    ,   CSTRING_RC_EXCEEDBORROWEDCAPACITY       /*!< Operation cannot procede because the current capacity would be exceeded and the cstring is borrowed */
};
#ifndef OPENRJ_DOCUMENTATION_SKIP_SECTION
typedef enum CSTRING_RC     CSTRING_RC;
#endif /* !OPENRJ_DOCUMENTATION_SKIP_SECTION */

/* /////////////////////////////////////////////////////////////////////////////
 * Typedefs
 */

/** \brief The cstring structure
 *
 * Represents a cstring as a string slice - length + pointer - along with the
 * allocated capacity, and flags controlling the allocation arena and other
 * characteristics.
 */
struct cstring_t
{
    size_t  len;        /*!< Number of characters.      */
    char    *ptr;       /*!< Pointer to the string.     */
    size_t  capacity;   /*!< Number of bytes available. */
    size_t  flags;      /*!< Flags. Currently unused.   */
};
#ifndef OPENRJ_DOCUMENTATION_SKIP_SECTION
typedef struct cstring_t    cstring_t;
#endif /* !OPENRJ_DOCUMENTATION_SKIP_SECTION */

/** \def cstring_t_DEFAULT
 * \brief Default initialised value for a cstring_t instance
 */
#define cstring_t_DEFAULT  { 0, NULL, 0, 0 }

/* /////////////////////////////////////////////////////////////////////////////
 * Custom settings
 */

#define CSTRING_F_TYPE_MASK                         (0x003f)
#define CSTRING_F_MEMORY_IS_INTERNAL_HEAP           (0x0000)    /*!< Memory is allocated by the heap function, and is owned by the cstring instance */
#define CSTRING_F_MEMORY_IS_BORROWED                (0x0001)    /*!< Memory is supplied external to the API, and may not be deallocated, moved or expanded. \note Implies CSTRING_F_MEMORY_IS_FIXED */
#define CSTRING_F_MEMORY_IS_FIXED                   (0x0002)    /*!< Memory is of fixed size, and any operation that requires expansion will fail */
#define CSTRING_F_MEMORY_IS_READONLY                (0x0004)    /*!< cstring instance may not be modified */
#define CSTRING_F_MEMORY_IS_CONST                   CSTRING_F_MEMORY_IS_READONLY /*!< Equivalent to CSTRING_F_MEMORY_IS_READONLY \deprecated */

#define CSTRING_F_ARENA_MASK                        (0xff00)
#define CSTRING_F_USE_REALLOC                       (0x0000)    /*!< Uses C standard library function realloc() for all memory (de-)allocation */
#if defined(WIN32)
# define CSTRING_F_USE_WIN32_GLOBAL_MEMORY          (0x0100)    /*!< Uses Win32 Global Memory API for all memory (de-)allocation (Windows only) */
# define CSTRING_F_USE_WIN32_PROCESSHEAP_MEMORY     (0x0200)    /*!< Uses Win32 Global Process Heap for all memory (de-)allocation (Windows only) */
# define CSTRING_F_USE_WIN32_COM_TASK_MEMORY        (0x0400)    /*!< Uses COM Task Allocator for all memory (de-)allocation (Windows only) */
#endif /* WIN32 */
#if defined(CSTRING_USE_SYNESIS_APIS)
# define CSTRING_F_USE_SYNESIS_HATOR                (0x1000)
#endif /* CSTRING_USE_SYNESIS_APIS */
#define CSTRING_F_USE_CUSTOMARENAFUNCTIONS          (0x2000)    /*!< Uses an externally supplied allocator function for all memory (de-)allocation (NOT CURRENTLY SUPPORTED) */

#define CSTRING_F_CUSTOMARENA_CC_CDECL              (0x0000)    /*!< Treats the externally supplied allocator function as having CDecl calling convention (de-)allocation (NOT CURRENTLY SUPPORTED) */
#define CSTRING_F_CUSTOMARENA_CC_STDCALL            (0x0040)    /*!< Treats the externally supplied allocator function as having StdCall calling convention (de-)allocation (NOT CURRENTLY SUPPORTED) */

/* /////////////////////////////////////////////////////////////////////////////
 * API functions
 */

/** \brief Returns a non-NULL nul-terminated character string describing the
 * given error code
 *
 * \param rc The error code. Must be one of the CSTRING_RC enumeration
 *
 * \return A non-NULL nul-terminated string
 */
CSTRING_EXTERN_C char const *cstring_error(     CSTRING_RC              rc);


/** \brief Initialises a cstring instance to a default form
 *
 * \note This must only be called on an uninitialised cstring instance
 *
 * \note This is equivalent to assigning an instance from cstring_t_DEFAULT.
 */
CSTRING_EXTERN_C CSTRING_RC cstring_init(       struct cstring_t        *pcs);

/** \brief Creates a cstring instance from a C-style string
 *
 * \param pcs The uninitialised cstring instance to be created
 * \param s Pointer to the C-style string to copy. May be NULL
 *
 * \note This must only be called on an uninitialised cstring instance
 *
 * \retval CSTRING_RC_OK the capacity was adjusted
 * \retval CSTRING_RC_OUTOFMEMORY insufficient memory to carry out the operation
 */
CSTRING_EXTERN_C CSTRING_RC cstring_create(     struct cstring_t        *pcs
                                        ,       char const              *s);

/** \brief Creates a cstring instance from a fixed number of characters
 *
 * \param pcs The uninitialised cstring instance to be created
 * \param s Pointer to the first character in the array from which to copy.
 *   Must not be NULL, unless \c cch is 0
 * \param cch The number of characters to be copied into the cstring instance
 *
 * \note This must only be called on an uninitialised cstring instance
 *
 * \retval CSTRING_RC_OK the capacity was adjusted
 * \retval CSTRING_RC_OUTOFMEMORY insufficient memory to carry out the operation
 */
CSTRING_EXTERN_C CSTRING_RC cstring_createLen(  struct cstring_t        *pcs
                                        ,       char const              *s
                                        ,       size_t                  cch);


/** \brief Creates a cstring instance with special characteristics from a C-style string
 *
 * \param pcs The uninitialised cstring instance to be created
 * \param s Pointer to the C-style string to copy. May be NULL
 * \param flags The flags defining the characteristics of the cstring instance
 * \param arena The memory buffer for borrowed (CSTRING_F_MEMORY_IS_BORROWED), 
 *   or a pointer to a custom memory arena. Note that custom memory arenas are
 *   not currently supported, and this parameter must be NULL unless the flag
 *   CSTRING_F_MEMORY_IS_BORROWED is specified
 * \param capacity The capacity of the buffer specified in \c arena, or
 *   otherwise ignored
 *
 * \note This must only be called on an uninitialised cstring instance
 *
 * \retval CSTRING_RC_OK the capacity was adjusted
 * \retval CSTRING_RC_OUTOFMEMORY insufficient memory to carry out the operation
 * \retval CSTRING_RC_INVALIDARENA an invalid arena specification was given in the \c flags parameter
 * \retval CSTRING_RC_CUSTOMARENANOTSUPPORTED A custom arena was specified, which is not yet supported
 */
CSTRING_EXTERN_C CSTRING_RC cstring_createEx(   struct cstring_t        *pcs
                                        ,       char const              *s
                                        ,       size_t                  flags
                                        ,       void                    *arena
                                        ,       size_t                  capacity);

/** \brief Creates a cstring instance with special characteristics from a fixed number of characters
 *
 * \param pcs The uninitialised cstring instance to be created
 * \param s Pointer to the first character in the array from which to copy.
 *   Must not be NULL, unless \c cch is 0
 * \param cch The number of characters to be copied into the cstring instance
 * \param flags The flags defining the characteristics of the cstring instance
 * \param arena The memory buffer for borrowed (CSTRING_F_MEMORY_IS_BORROWED), 
 *   or a pointer to a custom memory arena. Note that custom memory arenas are
 *   not currently supported, and this parameter must be NULL unless the flag
 *   CSTRING_F_MEMORY_IS_BORROWED is specified
 * \param capacity The capacity of the buffer specified in \c arena, or
 *   otherwise ignored
 *
 * \note This must only be called on an uninitialised cstring instance
 *
 * \retval CSTRING_RC_OK the capacity was adjusted
 * \retval CSTRING_RC_OUTOFMEMORY insufficient memory to carry out the operation
 * \retval CSTRING_RC_INVALIDARENA an invalid arena specification was given in the \c flags parameter
 * \retval CSTRING_RC_CUSTOMARENANOTSUPPORTED A custom arena was specified, which is not yet supported
 */
CSTRING_EXTERN_C CSTRING_RC cstring_createLenEx(struct cstring_t        *pcs
                                        ,       char const              *s
                                        ,       size_t                  cch
                                        ,       size_t                  flags
                                        ,       void                    *arena
                                        ,       size_t                  capacity);

/** \brief Releases resources and resets the attributes of the cstring instance
 *
 * \param pcs The cstring instance to be destroyed
 *
 * \note This must only be called on an initialised cstring instance, which is
 *   then rendered in the uninitialised state.
 *
 * \retval CSTRING_RC_OK the capacity was destroyed
 */
CSTRING_EXTERN_C CSTRING_RC cstring_destroy(    struct cstring_t        *pcs);


/** \brief Removes the allocated buffer from the cstring instance into a caller supplied pointer
 *
 * \param pcs the cstring instance whose buffer is to be retrieved
 * \param ppBuff pointer to a buffer pointer which will receive the yielded buffer
 *
 * \retval CSTRING_RC_OK the buffer was yielded
 * \retval CSTRING_RC_READONLY the cstring instance is marked readonly (CSTRING_F_MEMORY_IS_READONLY)
 * \retval CSTRING_RC_BORROWED the cstring instance is marked fixed (CSTRING_F_MEMORY_IS_BORROWED)
 */
CSTRING_EXTERN_C CSTRING_RC cstring_yield(      struct cstring_t        *pcs
                                        ,       char                    **ppBuff);

/** Sets the capacity of the cstring instance
 *
 * \param pcs The cstring instance to be adjusted
 * \param capacity The capacity to which the cstring instance should be adjusted
 *
 * \note If the specified capacity would cause the string contents to be 
 *   truncated, the value is adjusted to the minimum possible without 
 *   requiring truncation.
 *
 * \retval CSTRING_RC_OK the capacity was adjusted
 * \retval CSTRING_RC_READONLY the cstring instance is marked readonly (CSTRING_F_MEMORY_IS_READONLY)
 * \retval CSTRING_RC_EXCEEDFIXEDCAPACITY the cstring instances is marked fixed (CSTRING_F_MEMORY_IS_FIXED)
 * \retval CSTRING_RC_EXCEEDBORROWEDCAPACITY the cstring instances is marked borrowed (CSTRING_F_MEMORY_IS_BORROWED)
 * \retval CSTRING_RC_OUTOFMEMORY insufficient memory to carry out the operation
 */
CSTRING_EXTERN_C CSTRING_RC cstring_setCapacity(struct cstring_t        *pcs
                                        ,       size_t                  capacity);

/** \brief Assigns a C-style string to the cstring instance
 *
 * \param pcs The cstring to assign to 
 * \param s Pointer to the C-style string to assign. May be NULL
 *
 * \retval CSTRING_RC_OK the requested number of bytes were assigned to the cstring
 * \retval CSTRING_RC_READONLY the cstring instance is marked readonly (CSTRING_F_MEMORY_IS_READONLY)
 * \retval CSTRING_RC_EXCEEDFIXEDCAPACITY the cstring instances is marked fixed (CSTRING_F_MEMORY_IS_FIXED)
 * \retval CSTRING_RC_EXCEEDBORROWEDCAPACITY the cstring instances is marked borrowed (CSTRING_F_MEMORY_IS_BORROWED)
 * \retval CSTRING_RC_OUTOFMEMORY insufficient memory to carry out the operation
 *
 * \note The function is safe with respect to 'self-assignment', i.e, \c s can
 *   refer to (part of) the payload of \c pcs, and the function will still
 *   operate correctly to produce the logically desired result.
 */
CSTRING_EXTERN_C CSTRING_RC cstring_assign(     struct cstring_t        *pcs
                                        ,       char const              *s);

/** \brief Assigns a fixed number of characters to the cstring instance
 *
 * \param pcs The cstring to assign to 
 * \param s Pointer to the first character in the array from which to assign.
 *   Must not be NULL, unless \c cch is 0
 * \param cch The number of characters to assign
 *
 * \note If the source has embedded nul characters, they will be incorporated
 *   into the cstring contents, which means that calling strlen() on the payload
 *   pointer may give inconsistent results
 *
 * \retval CSTRING_RC_OK the requested number of bytes were assigned to the cstring
 * \retval CSTRING_RC_READONLY the cstring instance is marked readonly (CSTRING_F_MEMORY_IS_READONLY)
 * \retval CSTRING_RC_EXCEEDFIXEDCAPACITY the cstring instances is marked fixed (CSTRING_F_MEMORY_IS_FIXED)
 * \retval CSTRING_RC_EXCEEDBORROWEDCAPACITY the cstring instances is marked borrowed (CSTRING_F_MEMORY_IS_BORROWED)
 * \retval CSTRING_RC_OUTOFMEMORY insufficient memory to carry out the operation
 */
CSTRING_EXTERN_C CSTRING_RC cstring_assignLen(  struct cstring_t        *pcs
                                        ,       char const              *s
                                        ,       size_t                  cch);

/** \brief Copies the contents of one cstring instance to another
 *
 * \param pcs The cstring to assign to 
 * \param pcsSrc The cstring whose contents will be copied
 *
 * \retval CSTRING_RC_OK the requested number of bytes were copied to the cstring
 * \retval CSTRING_RC_READONLY the cstring instance is marked readonly (CSTRING_F_MEMORY_IS_READONLY)
 * \retval CSTRING_RC_EXCEEDFIXEDCAPACITY the cstring instances is marked fixed (CSTRING_F_MEMORY_IS_FIXED)
 * \retval CSTRING_RC_EXCEEDBORROWEDCAPACITY the cstring instances is marked borrowed (CSTRING_F_MEMORY_IS_BORROWED)
 * \retval CSTRING_RC_OUTOFMEMORY insufficient memory to carry out the operation
 */
CSTRING_EXTERN_C CSTRING_RC cstring_copy(       struct cstring_t        *pcs
                                        ,       struct cstring_t const  *pcsSrc);

/** \brief Appends a C-style string to the cstring instance
 *
 * \param pcs The cstring to append to 
 * \param s Pointer to the C-style string to append. May be NULL
 *
 * \retval CSTRING_RC_OK the requested number of bytes were appended to the cstring
 * \retval CSTRING_RC_READONLY the cstring instance is marked readonly (CSTRING_F_MEMORY_IS_READONLY)
 * \retval CSTRING_RC_EXCEEDFIXEDCAPACITY the cstring instances is marked fixed (CSTRING_F_MEMORY_IS_FIXED)
 * \retval CSTRING_RC_EXCEEDBORROWEDCAPACITY the cstring instances is marked borrowed (CSTRING_F_MEMORY_IS_BORROWED)
 * \retval CSTRING_RC_OUTOFMEMORY insufficient memory to carry out the operation
 */
CSTRING_EXTERN_C CSTRING_RC cstring_append(     struct cstring_t        *pcs
                                        ,       char const              *s);

/** \brief Appends a fixed number of characters to the cstring instance
 *
 * \param pcs The cstring to append to 
 * \param s Pointer to the first character in the array from which to append.
 *   Must not be NULL, unless \c cch is 0
 * \param cch The number of characters to append
 *
 * \note If the source has embedded nul characters, they will be incorporated
 *   into the cstring contents, which means that calling strlen() on the payload
 *   pointer may give inconsistent results
 *
 * \retval CSTRING_RC_OK the requested number of bytes were appended to the cstring
 * \retval CSTRING_RC_READONLY the cstring instance is marked readonly (CSTRING_F_MEMORY_IS_READONLY)
 * \retval CSTRING_RC_EXCEEDFIXEDCAPACITY the cstring instances is marked fixed (CSTRING_F_MEMORY_IS_FIXED)
 * \retval CSTRING_RC_EXCEEDBORROWEDCAPACITY the cstring instances is marked borrowed (CSTRING_F_MEMORY_IS_BORROWED)
 * \retval CSTRING_RC_OUTOFMEMORY insufficient memory to carry out the operation
 */
CSTRING_EXTERN_C CSTRING_RC cstring_appendLen(  struct cstring_t        *pcs
                                        ,       char const              *s
                                        ,       size_t                  cch);

/** \brief Truncates the cstring instance
 *
 * \param pcs The cstring to truncate
 * \param len The required length. Ignored if not less than the current length
 *
 * \note The cstring instance must not be readonly
 *
 * \retval CSTRING_RC_OK cstring instance contents were truncated (or the truncation length was out of bounds)
 * \retval CSTRING_RC_READONLY the cstring instance is marked readonly (CSTRING_F_MEMORY_IS_READONLY)
 */
CSTRING_EXTERN_C CSTRING_RC cstring_truncate(   struct cstring_t        *pcs
                                        ,       size_t                  len);

/** \brief Swaps the contents of the two cstring instances
 *
 * \param pcs1 the cstring instance whose contents will be swapped with \c pcs2
 * \param pcs2 the cstring instance whose contents will be swapped with \c pcs1
 *
 * \note Neither instance can be either readonly or fixed-and-borrowed
 *
 * \retval CSTRING_RC_OK string instance contents were swapped
 * \retval CSTRING_RC_READONLY one or both instances are marked readonly (CSTRING_F_MEMORY_IS_READONLY)
 * \retval CSTRING_RC_BORROWED one or both instances are marked fixed (CSTRING_F_MEMORY_IS_BORROWED)
 */
CSTRING_EXTERN_C CSTRING_RC cstring_swap(       struct cstring_t        *pcs1
                                        ,       struct cstring_t        *pcs2);

/* ////////////////////////////////////////////////////////////////////////// */

#endif /* CSTRING_INCL_CSTRING_H_CSTRING_INCL_H_CSTRING */

/* ////////////////////////////////////////////////////////////////////////// */
